#include <Servo.h>

// Servo
Servo myservo;

// Define color sensor pins
#define S0 4
#define S1 5
#define S2 6
#define S3 7
#define sensorOut 8

// Calibration Values
int redMin = 114; // Red minimum value
int redMax = 275; // Red maximum value
int greenMin = 250; // Green minimum value
int greenMax = 320; // Green maximum value
int blueMin = 192; // Blue minimum value
int blueMax = 242; // Blue maximum value

// Variables for Color Pulse Width Measurements
int redPW = 0;
int greenPW = 0;
int bluePW = 0;

// Variables for final Color values
int redValue;
int greenValue;
int blueValue;

// For motors
int leftMotorForward = 9;
int leftMotorBackward = 10;
int rightMotorForward = 11;
int rightMotorBackward = 12;

void setup() {
  // Set S0 - S3 as outputs
  pinMode(S0, OUTPUT);
  pinMode(S1, OUTPUT);
  pinMode(S2, OUTPUT);
  pinMode(S3, OUTPUT);
  
  // Set Sensor output as input
  pinMode(sensorOut, INPUT);
  
  // Set Frequency scaling to 20%
  digitalWrite(S0,HIGH);
  digitalWrite(S1,LOW);

  // Servo assign
  myservo.attach(13);
  myservo.write(90);
  
  // Setup Serial Monitor
  Serial.begin(9600);
}

void loop() {

  ColorCheck();
  Serial.print(ColorCheck());
  if(ColorCheck())
  {
    Forward();
  }
  if(!ColorCheck())
  {
    Brake();
    Search();
    delay(2000);
  }
  
}

bool ColorCheck()
{
  // Read Red value
  redPW = getRedPW();
  // Map to value from 0-255
  redValue = map(redPW, redMin,redMax,255,0);
  // Delay to stabilize sensor
  delay(100);
  
  // Read Green value
  greenPW = getGreenPW();
  // Map to value from 0-255
  greenValue = map(greenPW, greenMin,greenMax,255,0);
  // Delay to stabilize sensor
  delay(100);
  
  // Read Blue value
  bluePW = getBluePW();
  // Map to value from 0-255
  blueValue = map(bluePW, blueMin,blueMax,255,0);
  // Delay to stabilize sensor
  delay(100);
  
  // Print output to Serial Monitor
  Serial.print("Red = ");
  Serial.print(redValue);
  Serial.print(" - Green = ");
  Serial.print(greenValue);
  Serial.print(" - Blue = ");
  Serial.println(blueValue);

  // Check if color is green and returns
  if (greenValue > redValue && greenValue > blueValue)
  {
    return true;
  } else {
    return false; 
  }
}

// Function to read Red Pulse Widths
int getRedPW() {

  // Set sensor to read Red only
  digitalWrite(S2,LOW);
  digitalWrite(S3,LOW);
  // Define integer to represent Pulse Width
  int PW;
  // Read the output Pulse Width
  PW = pulseIn(sensorOut, LOW);
  // Return the value
  return PW;

}

// Function to read Green Pulse Widths
int getGreenPW() {

  // Set sensor to read Green only
  digitalWrite(S2,HIGH);
  digitalWrite(S3,HIGH);
  // Define integer to represent Pulse Width
  int PW;
  // Read the output Pulse Width
  PW = pulseIn(sensorOut, LOW);
  // Return the value
  return PW;

}

// Function to read Blue Pulse Widths
int getBluePW() {

  // Set sensor to read Blue only
  digitalWrite(S2,LOW);
  digitalWrite(S3,HIGH);
  // Define integer to represent Pulse Width
  int PW;
  // Read the output Pulse Width
  PW = pulseIn(sensorOut, LOW);
  // Return the value
  return PW;

}

// Function to search line is lost
void Search()
{
  Brake();
  
  bool goRight = LookRight();
  delay(1000);
  
  if (goRight)
  {
    Right();
    ColorCheck;
    while (!ColorCheck)
    {
      Right();
      ColorCheck;
    }
  }
  if (!goRight)
  {
    bool goLeft = LookLeft();
    delay(1000);
    
    if (goLeft)
    {
      Left();
      ColorCheck;
      while (!ColorCheck)
      {
        Left();
        ColorCheck;
      }
    }
  }

  
}

bool LookLeft()
{
  
  int pos;
  for (pos = 90; pos < 180; pos += 10)
  {
      myservo.write(pos);
      ColorCheck();
      if (ColorCheck())
      {
        myservo.write(90);
        return true;
      }
      delay(15);
  }
  return false;
}

bool LookRight()
{
  
  int pos;
  for (pos = 90; pos >= 1; pos -= 10)
  {
      myservo.write(pos);
      ColorCheck();
      if (ColorCheck())
      {
        myservo.write(90);
        return true;
      }
      delay(15);
  }
  return false;
}

// Function to move the car forward
void Forward()
{
  digitalWrite(leftMotorForward, HIGH);
  digitalWrite(rightMotorForward, HIGH);
  delay(500);
  digitalWrite(leftMotorForward, LOW);
  digitalWrite(rightMotorForward, LOW);
}

// Function to turn the car left
void Left()
{
  digitalWrite(leftMotorForward, LOW);
  digitalWrite(rightMotorForward, HIGH);
  delay(500);
  digitalWrite(leftMotorForward, LOW);
  digitalWrite(rightMotorForward, LOW);
}

//function to turn the car right
void Right()
{
  digitalWrite(leftMotorForward, HIGH);
  digitalWrite(rightMotorForward, LOW);
  delay(500);
  digitalWrite(leftMotorForward, LOW);
  digitalWrite(rightMotorForward, LOW);
}

// Function to move the car in reverse
void Reverse()
{
  digitalWrite(leftMotorBackward, HIGH);
  digitalWrite(rightMotorBackward, HIGH);
}

// Function to make the car stop
void Brake()
{
  digitalWrite(leftMotorForward, LOW);
  digitalWrite(rightMotorForward, LOW);
}
